import { ISwaggerInfo, ISwaggerExternalDocs } from "./i-swagger";
export interface ISwaggerBuildDefinitionModelProperty {
    /**
     * Define type of property. Example: SwaggerDefinitionConstant.Definition.Property.Type.STRING
     */
    type: string;
    /**
     * Define format of property. Example: SwaggerDefinitionConstant.Definition.Property.Format.INT_64
     * Optional.
     */
    format?: string;
    /**
     * Define if property is required.
     * Optional. Default is false.
     */
    required?: boolean;
}
export interface ISwaggerBuildDefinitionModel {
    /**
     * Define all properties of model.
     */
    properties: {
        [key: string]: ISwaggerBuildDefinitionModelProperty;
    };
}
export interface ISwaggerBuildDefinition {
    /**
     * Base URL for all API.
     * Optional. Default is "/".
     */
    basePath?: string;
    /**
     * Version Open API
     * Optional.
     */
    openapi?: string;
    /**
     * Metadata.
     */
    info: ISwaggerInfo;
    /**
     * Define the MIME types supported by the API for consumes. The root-level definition can be overridden in individual operations.
     * Optional. Default is SwaggerDefinition.Consume.JSON = "application/json".
     */
    consumes?: string[];
    /**
     * Define the MIME types supported by the API for produces. The root-level definition can be overridden in individual operations.
     * Optional. Default is SwaggerDefinition.Produce.JSON = "application/json".
     */
    produces?: string[];
    /**
     * Define schemes.
     * Optional. Default is SwaggerDefinition.Scheme.HTTP = "http".
     */
    schemes?: string[];
    /**
     * Define host.
     * Optional.
     */
    host?: string;
    /**
     * Define All Definitions.
     * Optional.
     */
    models?: {
        [key: string]: ISwaggerBuildDefinitionModel;
    };
    /**
     * Define external doc
     * Optional.
     */
    externalDocs?: ISwaggerExternalDocs;
}
export declare function build(buildDefinition: ISwaggerBuildDefinition): void;
